import { Component, OnInit, Output, EventEmitter } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatDialogRef } from '@angular/material/dialog';
import { HttpClient, HttpHeaders } from '@angular/common/http';

@Component({
  selector: 'app-create-organizational-unit',
  templateUrl: './create-organizational-unit.component.html',
  styleUrls: ['./create-organizational-unit.component.css']
})
export class CreateOrganizationalUnitComponent implements OnInit {
  isLinear = true;
  generalFormGroup: FormGroup;
  additionalInfoFormGroup: FormGroup;
  networkSettingsFormGroup: FormGroup;
  classroomInfoFormGroup: FormGroup; 
  types: string[] = ['organizational-unit', 'classrooms-group', 'classroom', 'clients-group'];
  parentUnits: any[] = []; 

  @Output() unitAdded = new EventEmitter(); 

  constructor(
    private _formBuilder: FormBuilder,
    private dialogRef: MatDialogRef<CreateOrganizationalUnitComponent>,
    private http: HttpClient 
  ) {
    this.generalFormGroup = this._formBuilder.group({
      name: ['', Validators.required],
      parent: [''],
      description: [''],
      type: ['', Validators.required]
    });
    this.additionalInfoFormGroup = this._formBuilder.group({
      comments: [''],
    });
    this.networkSettingsFormGroup = this._formBuilder.group({
      proxy: [''],
      dns: [''],
      netmask: [''],
      router: [''],
      ntp: [''],
      p2pMode: [''],
      p2pTime: [0, Validators.min(0)],
      mcastIp: [''],
      mcastSpeed: [0, Validators.min(0)],
      mcastPort: [0, Validators.min(0)],
      mcastMode: [''],
      menu: ['', Validators.pattern('https?://.+')],
      hardwareProfile: ['', Validators.pattern('https?://.+')],
      validation: [false]
    });
    this.classroomInfoFormGroup = this._formBuilder.group({
      location: [''],
      projector: [false],
      board: [false],
      capacity: [0, Validators.min(0)],
      photo: ['', Validators.pattern('https?://.+')] 
    });
  }

  ngOnInit() {
    this.loadParentUnits(); 
  }

  loadParentUnits() {
    const url = 'http://127.0.0.1:8080/organizational-units?page=1&itemsPerPage=30';

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
      },
      error => {
        console.error('Error fetching parent units:', error);
      }
    );
  }

  private cleanFormValues(formGroup: FormGroup): any {
    const cleanedValues: any = {};
    Object.keys(formGroup.controls).forEach(key => {
      const value = formGroup.get(key)?.value;
      if (value !== '' && value !== 0) {
        cleanedValues[key] = value;
      }
    });
    return cleanedValues;
  }

  onSubmit() {
    if (this.generalFormGroup.valid && this.additionalInfoFormGroup.valid && this.networkSettingsFormGroup.valid && (this.generalFormGroup.value.type !== 'classroom' || this.classroomInfoFormGroup.valid)) {
      const generalFormValues = this.cleanFormValues(this.generalFormGroup);
      const additionalInfoFormValues = this.cleanFormValues(this.additionalInfoFormGroup);
      const networkSettingsFormValues = this.cleanFormValues(this.networkSettingsFormGroup);
      const classroomInfoFormValues = this.cleanFormValues(this.classroomInfoFormGroup);

      const formData: any = {
        ...generalFormValues,
        comments: additionalInfoFormValues.comments,
        networkSettings: { ...networkSettingsFormValues }
      };

      if (this.generalFormGroup.value.type === 'classroom') {
        formData.location = classroomInfoFormValues.location;
        formData.projector = classroomInfoFormValues.projector;
        formData.board = classroomInfoFormValues.board;
        if (classroomInfoFormValues.capacity !== undefined) {
          formData.capacity = classroomInfoFormValues.capacity;
        }
        if (classroomInfoFormValues.photo !== undefined) {
          formData.photo = classroomInfoFormValues.photo;
        }
      }
      
      console.log('POST data:', formData);
      const postUrl = 'http://127.0.0.1:8080/organizational-units';
      const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

      this.http.post<any>(postUrl, formData, { headers }).subscribe(
        response => {
          console.log('POST successful:', response);
          this.unitAdded.emit();
          this.dialogRef.close(); 
        },
        error => {
          console.error('Error al realizar POST:', error);
        }
      );
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }
}
