import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Component, EventEmitter, Inject, OnInit, Output } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { CreateOrganizationalUnitComponent } from '../create-organizational-unit/create-organizational-unit.component';

@Component({
  selector: 'app-edit-organizational-unit',
  templateUrl: './edit-organizational-unit.component.html',
  styleUrl: './edit-organizational-unit.component.css'
})
export class EditOrganizationalUnitComponent implements OnInit {
  isLinear = true;
  generalFormGroup: FormGroup;
  additionalInfoFormGroup: FormGroup;
  networkSettingsFormGroup: FormGroup;
  types: string[] = ['organizational-unit', 'classrooms-group', 'classroom', 'clients-group'];
  parentUnits: any[] = []; // Array to store parent units fetched from API
  isEditMode: boolean; // Flag to check if it's edit mode

  @Output() unitAdded = new EventEmitter(); // Event emitter to notify parent component about unit addition

  constructor(
    private _formBuilder: FormBuilder,
    private dialogRef: MatDialogRef<CreateOrganizationalUnitComponent>,
    private http: HttpClient, // Inject HttpClient for HTTP requests
    @Inject(MAT_DIALOG_DATA) public data: any // Inject data for edit mode
  ) {
    this.isEditMode = !!data?.uuid; // Check if uuid is passed to determine edit mode

    this.generalFormGroup = this._formBuilder.group({
      name: ['', Validators.required],
      parent: [''],
      description: [''],
      type: ['', Validators.required]
    });

    this.additionalInfoFormGroup = this._formBuilder.group({
      comments: [''],
    });

    this.networkSettingsFormGroup = this._formBuilder.group({
      proxy: [''],
      dns: [''],
      netmask: [''],
      router: [''],
      ntp: [''],
      p2pMode: [''],
      p2pTime: [0, Validators.min(0)],
      mcastIp: [''],
      mcastSpeed: [0, Validators.min(0)],
      mcastPort: [0, Validators.min(0)],
      mcastMode: [''],
      menu: ['', Validators.pattern('https?://.+')],
      hardwareProfile: ['', Validators.pattern('https?://.+')],
      validation: [false]
    });

    if (this.isEditMode) {
      this.loadData(data.uuid);
    }
  }

  ngOnInit() {
    this.loadParentUnits(); // Load parent units when component initializes
  }

  loadParentUnits() {
    const url = 'http://127.0.0.1:8080/organizational-units?page=1&itemsPerPage=30';

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
      },
      error => {
        console.error('Error fetching parent units:', error);
      }
    );
  }

  loadData(uuid: string) {
    const url = `http://127.0.0.1:8080/organizational-units/${uuid}`;

    this.http.get<any>(url).subscribe(
      data => {
        this.generalFormGroup.patchValue({
          name: data.name,
          parent: data.parent ? data.parent['@id'] : '',
          description: data.description,
          type: data.type
        });
        this.additionalInfoFormGroup.patchValue({
          comments: data.comments
        });
        this.networkSettingsFormGroup.patchValue({
          proxy: data.proxy,
          dns: data.dns,
          netmask: data.netmask,
          router: data.router,
          ntp: data.ntp,
          p2pMode: data.p2pMode,
          p2pTime: data.p2pTime,
          mcastIp: data.mcastIp,
          mcastSpeed: data.mcastSpeed,
          mcastPort: data.mcastPort,
          mcastMode: data.mcastMode,
          menu: data.menu,
          hardwareProfile: data.hardwareProfile,
          validation: data.validation
        });
      },
      error => {
        console.error('Error fetching data for edit:', error);
      }
    );
  }

  onSubmit() {
    if (this.generalFormGroup.valid && this.additionalInfoFormGroup.valid && this.networkSettingsFormGroup.valid) {
      const parentValue = this.generalFormGroup.value.parent;

      const formData = {
        name: this.generalFormGroup.value.name,
        parent: parentValue || null,
        description: this.generalFormGroup.value.description,
        comments: this.additionalInfoFormGroup.value.comments,
        type: this.generalFormGroup.value.type,
        ...this.networkSettingsFormGroup.value
      };

      if (this.isEditMode) {
        // Edit mode: Send PUT request to update the unit
        const putUrl = `http://127.0.0.1:8080/organizational-units/${this.data.uuid}`;
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

        this.http.put<any>(putUrl, formData, { headers }).subscribe(
          response => {
            console.log('PUT successful:', response);
            this.unitAdded.emit();
            this.dialogRef.close();
          },
          error => {
            console.error('Error al realizar PUT:', error);
          }
        );
      } else {
        // Create mode: Send POST request to create a new unit
        const postUrl = 'http://127.0.0.1:8080/organizational-units';
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

        this.http.post<any>(postUrl, formData, { headers }).subscribe(
          response => {
            console.log('POST successful:', response);
            this.unitAdded.emit();
            this.dialogRef.close();
          },
          error => {
            console.error('Error al realizar POST:', error);
          }
        );
      }
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }
}
